#pragma once

#include "stdafx.h"

//#ifdef _WIN32
//#include <windows.h>
//#include <winbase.h>
//#endif

//#include <WinDef.h>

#include "SoftwareConfig.h"
#include "drivercommon.h"
#include "mcrcommon.h"
#include "IntApi.h"


void AcquireApiMutex(void);
void ReleaseApiMutex(void);

// DLL definitions

#define DLL_VERSION_MAJOR			1
#define DLL_VERSION_MINOR			0
#define DLL_VERSION_REVISION		0

// Macros

// bit mapped flags
#define ALARM_PER_SECOND         0x0001
#define ALARM_FRAME_START        0x0002
#define ALARM_FRAME_END          0x0004
#define ALARM_MINUTE             0x0008
#define ALARM_LOCK_GAIN          0x0010
#define ALARM_LOCK_LOSS          0x0020
#define RECALIBRATE_NOW          0x0040






//
// a structure to contain a time code frame read and
// associated statuses
//

typedef struct _TIME_CODE_FRAME_REV_A
{
	UCHAR			tcBuffer[8];	// Buffer for timecode to be read into
	                                // This is not raw timecode. Some decoding and reformatting has been done.
									// See the IRIG and SMPTE structures in mcrcommon.h for the formatting
	                                // of the time code.
	UCHAR			tcChecksum;		// timecode checksum
	UCHAR			tcStatus;		// time/time code status

    LARGE_INTEGER   intPerfCount;   // performance counter as of last interrupt

    DWORD           hwStatus;		// hardware status

	UCHAR           ucLockStatus;   // contains the overall lock status (includes GPS, time code, HSO, RTC)

	UCHAR           ucReserved[8];

} TIME_CODE_FRAME_REV_A, *PTIME_CODE_FRAME_REV_A;



// time code reader hardware and kernel-mode driver status information

typedef struct _DEVICE_INFO
{
	// model ID of MCR card in use, one of TC_PRODUCT_xxx
	DWORD		devType;

	// model version of MCR card in use
	DWORD		devVersion;

	// operation mode of MCR card
	DWORD		devMode; // not implemented

	// status of MCR card, one of HW_STATUS_xxx
	DWORD		devStatus;

	// MCR card kernel-mode driver version
	MCRDRVVER	driverVersion;

	// MCR card kernel-mode driver status, one of TC_STATUS_xxx
   	DWORD		tcStatus;

	// GPS status (if time code source has SMPTE, Leitch encoding, with Masterclock, Inc proprietary control bits)
	DWORD		dwTimeCodeGpsStatus;

	// frame rate of incoming time code (only used for time codes with sub-second frame rates)
	WORD		frameRate;

} DEVICE_INFO, *PDEVICE_INFO;

typedef struct _CALLBACK_NODE
{
	// callback control
	PMCR_ASYNC_CALLBACK_ROUTINE	cbFunction;
	DWORD						cbThreadId;
	HANDLE						cbThreadHandle;
	DWORD						cbHooksMask;
	PVOID						next;
	PVOID						prev;
	BOOL						locked;

	// data for each thread's callback function
	MCR_TIME_REV_A				timeData;
} CALLBACK_NODE, *PCALLBACK_NODE;

extern DWORD ReadAndProcessTimeFrame(TIME_CODE_FRAME_REV_A *ptimeFrameToReturn, LARGE_INTEGER *pliPerfCountIn, MCR_TIME_REV_A *mcrTime);
//extern DWORD GetTCTypeFromTCStatus(UINT8 tcStatus);

extern DWORD			g_dwTcType;
extern DWORD			g_dllInitStatus;
extern DEVICE_INFO		g_devInfo;
extern PCALLBACK_NODE	g_cbListHead;
//extern HANDLE			mcrInterruptEvent;

#define DLL_IS_PRESENT 0x1155AACCul
// interprocess DLL instance shared data and handling functions
typedef struct {
	DWORD     dwDllIsPresent;
	HINSTANCE hMcrDllInstanceLib;
	volatile LONG  ldllApiMutex;			// We cannot use a true mutex (ie. CreateMutex() and WaitForSingleObject() )
											// Simulate one using this flag.
	volatile LONG  ldllSerialEventMutex;	// We cannot use a true mutex (ie. CreateMutex() and WaitForSingleObject() )
											// Simulate one using this flag.

	DWORD		dllInstanceCnt;

	// Critical section for accessing Callback list data
	CRITICAL_SECTION csCBListData;

	// Mutex to control access to Callback counts
	HANDLE	    hCBCountMutex;
	DWORD		dllCallbackPerSecEnableCnt;
	DWORD		dllCallbackStartFrameEnableCnt;
	DWORD		dllCallbackEndFrameEnableCnt;
	DWORD		dllCallbackPerMinEnableCnt;
	DWORD		dllCallbackTCDetectEnableCnt;
	DWORD		dllCallbackTCLossEnableCnt;

	// Mutex to control access to LastInt status and perf count.
	HANDLE	      hLastIntMutex;
	UCHAR		  dllLastIntReason;
} MC_DLL_SHARED;

extern MC_DLL_SHARED g_srDllShared;

extern DWORD fnRemoveCallbackEnableCounts(DWORD cbToRemove);
extern DWORD fnAddCallbackEnableCounts(DWORD cbToAdd);
