#include "StdAfx.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <search.h>

#include "mcrcommon.h"
#include "SoftwareConfig.h"
#include "DebugOutput.h"

//#pragma managed

// registry key names
#define REGISTRY_TIME_ZONE_KEY          (_T("Registry Time Zone Key"))
#define REGISTRY_TIME_ZONE_DESC         (_T("Source Time Zone Description"))
#define REGISTRY_TIME_ZONE_INFO         (_T("Source Time Zone"))
#define REGISTRY_DST_ENCODED            (_T("DST Encoded"))
#define REGISTRY_TIME_CODE_SOURCE_SMPTE (_T("Time Code Source Smpte"))
#define REGISTRY_TIME_CODE_SOURCE_IRIG  (_T("Time Code Source Irig"))
#define REGISTRY_FREEWHEEL_SYNC         (_T("Freewheel Synchronize"))
#define REGISTRY_FREEWHEEL_DISP         (_T("Freewheel Display"))
#define REGISTRY_TIME_CODE_LOSS         (_T("Time Code Loss Notify"))
#define REGISTRY_PCI_COM_PORT           (_T("PCI COM Port Number"))
#define REGISTRY_PCI_VIRTUAL_COM_PORT   (_T("PCI Virtual COM Port Number"))
#define REGISTRY_AGGRESSIVE_TIME_SET    (_T("Aggressive Time Set"))
#define REGISTRY_CRUDE_TIME_SET         (_T("Use Crude Time Set"))
#define REGISTRY_NMEA_MESSAGES_BIT_MAP  (_T("Nmea Messages Bit Map"))
#define REGISTRY_NMEA_BAUD_RATE         (_T("Nmea Baud Rate"))
#define REGISTRY_NMEA_DATA_BITS         (_T("Nmea Data Bits"))
#define REGISTRY_NMEA_PARITY            (_T("Nmea Parity"))
#define REGISTRY_NMEA_STOP_BITS         (_T("Nmea Stop Bits"))
#define REGISTRY_NMEA_COM_PORT_NUMBER   (_T("Nmea Com Port Number"))



int tzCompare(const void *arg1, const void *arg2);


CSoftwareConfig::CSoftwareConfig(CDebugOutput *cDebugOutputIn)
{
	cDebugConfig = cDebugOutputIn;

	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("Load Config class"));
	#endif

	GetKeyDefaults(&m_srKeys);

	// This will verify the keys exist in the registry. If they do not, the key will be created and set to the default value
	VerifyRegistryKeys();

	LoadConfigFromRegistry();

	GetRegistryTimeZones();
}


//
//		Name:				::GetImagePath()
//
//		Description:		Assumes the registry keys exist and have been initialized.
//

//void CSoftwareConfig::GetImagePath(TCHAR *destBuff, int iBuffSize)
//{
//	TCHAR	tcParamPath[256];
//	HKEY	rKey;
//
//	DWORD	paramType;
//	DWORD	paramSize;
//	LSTATUS lStatus;
//
//	GetRegistryPath(tcParamPath, sizeof(tcParamPath)/sizeof(TCHAR));
//
//	lStatus = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
//					tcParamPath,
//					0,
//					KEY_READ,
//					&rKey);
//	if (lStatus != ERROR_SUCCESS)
//	{
//		#ifdef _DEBUG_ERROR
//		cDebugConfig->DebugMsg(_T("!!!GetImagePath reg open Key failed"));
//		#endif
//	}
//
//
//	paramType = REG_EXPAND_SZ;
//	paramSize = iBuffSize-1;
//
//	lStatus = RegQueryValueEx(rKey,
//					_T("ImagePath"),
//					NULL,
//					&paramType,
//					(unsigned char *)destBuff,
//					&paramSize);
//
//	if (lStatus != ERROR_SUCCESS)
//	{
//		#ifdef _DEBUG_ERROR
//		cDebugConfig->DebugMsg(_T("!!!GetImagePath reg query Key failed"));
//		#endif
//	}
//	RegCloseKey(rKey);
//}

//
//		Name:				::LoadConfigFromRegistry()
//
//		Description:		Loads software configuration settings from the Windows NT registry
//							from the \SYSTEM\CurrentControlSet\Services\ourservicename\Parameters
//							key.
//
//		Note:				Inability to load any of the undocumented settings does not fail the
//							configuration load.
//
//		Returns:			FALSE if error reading the registry.
//

int CSoftwareConfig::LoadConfigFromRegistry()
{
	HKEY	paramKey;
	DWORD	paramType;
	DWORD	paramSize;
	TCHAR	tcParamPath[256];

	BOOL    bIsFailed;

	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("Load Config from registry"));
	#endif

	bIsFailed = FALSE;

	GetRegistryPath(tcParamPath, sizeof(tcParamPath)/sizeof(TCHAR));

	wcscat_s(tcParamPath,sizeof(tcParamPath)/sizeof(TCHAR), _T("\\Parameters"));

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,tcParamPath,0,KEY_READ,&paramKey) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;

		VerifyRegistryKeys();

		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config path error"));
		#endif
	}

	paramType = REG_SZ;
	paramSize = sizeof(m_srKeys.tzRegistryKey);

	if (RegQueryValueEx(paramKey,
						REGISTRY_TIME_ZONE_KEY,
						NULL,
						&paramType,
						(BYTE *)m_srKeys.tzRegistryKey,
						&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
        #ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config time zone key error"));
        #endif
	}

	paramType = REG_SZ;
	paramSize = sizeof(m_srKeys.tzDesc);

	if (RegQueryValueEx(paramKey,
						REGISTRY_TIME_ZONE_DESC,
						NULL,
						&paramType,
						(BYTE *)m_srKeys.tzDesc,
						&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config time zone description error"));
		#endif
	}

	paramType = REG_BINARY;
	paramSize = sizeof(m_srKeys.tzInfo);

	if (RegQueryValueEx(paramKey,
						REGISTRY_TIME_ZONE_INFO,
						NULL,
						&paramType,
						(BYTE *)&m_srKeys.tzInfo,
						&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config time zone info error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
					REGISTRY_DST_ENCODED,
					NULL,
					&paramType,
					(BYTE *)&m_srKeys.dstEncoded,
					&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config dst encoded error error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_TIME_CODE_SOURCE_SMPTE,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.timeCodeSourceSmpte,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config time code source SMPTE error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_TIME_CODE_SOURCE_IRIG,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.timeCodeSourceIrig,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config time code source IRIG error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
					REGISTRY_FREEWHEEL_SYNC,
					NULL,
					&paramType,
					(BYTE *)&m_srKeys.freewheelSynchronize,
					&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config freewheel sync error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
					REGISTRY_FREEWHEEL_DISP,
					NULL,
					&paramType,
					(BYTE *)&m_srKeys.freewheelDisplay,
					&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config freewheel display error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
					REGISTRY_TIME_CODE_LOSS,
					NULL,
					&paramType,
					(BYTE *)&m_srKeys.tcLossNotify,
					&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config tc loss notify error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_PCI_COM_PORT,
					NULL,
					&paramType,
		            (BYTE *)&m_srKeys.pciComPort,
					&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config PCI COM port error"));
		#endif
	}


	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_PCI_VIRTUAL_COM_PORT,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.pciNmeaVirtualComPort,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config PCI Virtual COM port error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_NMEA_MESSAGES_BIT_MAP,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.dwNmeaMessagesBitMap,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config NMEA messages bit map error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_NMEA_BAUD_RATE,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.dwNmeaBaudrate,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config NMEA baud rate error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_NMEA_DATA_BITS,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.dwNmeaDataBits,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config NMEA data bits error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_NMEA_PARITY,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.dwNmeaParity,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config NMEA parity error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_NMEA_STOP_BITS,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.dwNmeaStopBits,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config NMEA stop bits error"));
		#endif
	}



	paramType = REG_DWORD;
	paramSize = 4;

	if (RegQueryValueEx(paramKey,
		REGISTRY_NMEA_COM_PORT_NUMBER,
		NULL,
		&paramType,
		(BYTE *)&m_srKeys.dwNmeaComPortNumber,
		&paramSize) != ERROR_SUCCESS)
	{
		bIsFailed = TRUE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Load Config NMEA COM port number error"));
		#endif
	}

	if (bIsFailed)
	{
		VerifyRegistryKeys();
	}


	// note that the remaining configuration items are undocumented or special-purpose
	// configuration and configuration load does not fail if they are missing

	//paramType = REG_DWORD;
	//paramSize = 4;

	//if (RegQueryValueEx(paramKey,
	//				REGISTRY_AGGRESSIVE_TIME_SET,
	//				NULL,
	//				&paramType,
	//				(BYTE *)&m_srKeys.setTimeAggressive,
	//				&paramSize) != ERROR_SUCCESS)
	//{
	//	m_srKeys.setTimeAggressive = 0L;
	//	#ifdef _DEBUG_ERROR
	//	cDebugConfig->DebugMsg(_T("!!!Load Config aggressive set error"));
	//	#endif
	//}

	//paramType = REG_DWORD;
	//paramSize = 4;

	//if (RegQueryValueEx(paramKey,
	//				REGISTRY_CRUDE_TIME_SET,
	//				NULL,
	//				&paramType,
	//				(BYTE *)&m_srKeys.useCrudeTimeSet,
	//				&paramSize) != ERROR_SUCCESS)
	//{
	//	m_srKeys.useCrudeTimeSet = 0L;
	//	#ifdef _DEBUG_ERROR
	//	cDebugConfig->DebugMsg(_T("!!!Load Config crude set error"));
	//	#endif
	//}

	RegCloseKey(paramKey);

	return(TRUE);
}


//
//		Name:				::SaveConfigToRegistry()
//
//		Description:		Saves software configuration members to the Windows NT registry
//							under the \SYSTEM\CurrentControlSet\Services\ourservicename\Parameters
//							key.
//
//		Note:				We don't write changes for the undocumented settings.  This is
//							a process to be performed only manually.
//
//		Returns:			FALSE if error writing to the registry.
//

BOOL CSoftwareConfig::SaveConfigToRegistry()
{
	HKEY	paramKey;
	TCHAR	tcParamPath[256];

	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("Save Config to registry"));
	#endif

	GetRegistryPath(tcParamPath, sizeof(tcParamPath)/sizeof(TCHAR));

	wcscat_s(tcParamPath,sizeof(tcParamPath)/sizeof(TCHAR), _T("\\Parameters"));

	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
		             tcParamPath,
					 0,
					 KEY_SET_VALUE,
					 &paramKey) != ERROR_SUCCESS)
	{
		return FALSE;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Save Config path error"));
		#endif
	}

	RegSetValueEx( paramKey,
						REGISTRY_TIME_ZONE_KEY,
						0,
						REG_SZ,
						(BYTE *)m_srKeys.tzRegistryKey,
						(DWORD)_tcslen(m_srKeys.tzRegistryKey)*sizeof(TCHAR) + sizeof(TCHAR));

	RegSetValueEx( paramKey,
						REGISTRY_TIME_ZONE_DESC,
						0,
						REG_SZ,
						(BYTE *)m_srKeys.tzDesc,
						(DWORD)_tcslen(m_srKeys.tzDesc)*sizeof(TCHAR) + sizeof(TCHAR));

	RegSetValueEx(	paramKey,
						REGISTRY_TIME_ZONE_INFO,
						0,
						REG_BINARY,
						(BYTE *)&m_srKeys.tzInfo,
						(DWORD)sizeof(m_srKeys.tzInfo));

	RegSetValueEx( paramKey,
						REGISTRY_DST_ENCODED,
						0,
						REG_DWORD,
						(BYTE *)&m_srKeys.dstEncoded,
						(DWORD)sizeof(DWORD));

	RegSetValueEx( paramKey,
		                REGISTRY_TIME_CODE_SOURCE_SMPTE,
		                0,
		                REG_DWORD,
		                (BYTE *)&m_srKeys.timeCodeSourceSmpte,
		                sizeof(DWORD));

	RegSetValueEx( paramKey,
						REGISTRY_TIME_CODE_SOURCE_IRIG,
						0,
						REG_DWORD,
						(BYTE *)&m_srKeys.timeCodeSourceIrig,
						(DWORD)sizeof(DWORD));

	RegSetValueEx(	paramKey,
						REGISTRY_FREEWHEEL_SYNC,
						0,
						REG_DWORD,
						(BYTE *)&m_srKeys.freewheelSynchronize,
						(DWORD)sizeof(DWORD));

	RegSetValueEx(	paramKey,
						REGISTRY_FREEWHEEL_DISP,
						0,
						REG_DWORD,
						(BYTE *)&m_srKeys.freewheelDisplay,
						(DWORD)sizeof(DWORD));

	RegSetValueEx(	paramKey,
						REGISTRY_TIME_CODE_LOSS,
						0,
						REG_DWORD,
						(BYTE *)&m_srKeys.tcLossNotify,
						(DWORD)sizeof(DWORD));

	RegSetValueEx(	paramKey,
						REGISTRY_PCI_COM_PORT,
						0,
						REG_DWORD,
						(BYTE *)&m_srKeys.pciComPort,
						(DWORD)sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_PCI_VIRTUAL_COM_PORT,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.pciNmeaVirtualComPort,
		(DWORD)sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_NMEA_MESSAGES_BIT_MAP,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.dwNmeaMessagesBitMap,
		sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_NMEA_BAUD_RATE,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.dwNmeaBaudrate,
		sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_NMEA_DATA_BITS,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.dwNmeaDataBits,
		sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_NMEA_PARITY,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.dwNmeaParity,
		sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_NMEA_STOP_BITS,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.dwNmeaStopBits,
		sizeof(DWORD));


	RegSetValueEx(	paramKey,
		REGISTRY_NMEA_COM_PORT_NUMBER,
		0,
		REG_DWORD,
		(BYTE *)&m_srKeys.dwNmeaComPortNumber,
		sizeof(DWORD));

	RegCloseKey(paramKey);

	return TRUE;
}



//
//		Name:				::GetRegistryPath()
//
//		Description:		Gets the registry path.
//

void CSoftwareConfig::GetRegistryPath(TCHAR *pPath, int iPathLength)
{
		swprintf_s(pPath,iPathLength,_T("SOFTWARE\\Masterclock\\%s"),
			REGISTRY_SOURCE_NAME);
}

//
//		Name:			::GetKeyDefaults()
//
//		Description:	Set the structure to the default key values
//

void CSoftwareConfig::GetKeyDefaults(REGISTRY_KEYS *psrMyKeys)
{
	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("Get config key defaults"));
	#endif

	memset(psrMyKeys, 0, sizeof(REGISTRY_KEYS));

	psrMyKeys->dstEncoded              = FALSE;
	psrMyKeys->timeCodeSourceSmpte     = TC_SOURCE_DATE_LEITCH_FORMAT;
	psrMyKeys->timeCodeSourceIrig      = TC_SOURCE_DATE_DOY_YY;
	psrMyKeys->freewheelSynchronize    = TRUE;
	psrMyKeys->freewheelDisplay        = TRUE;
	psrMyKeys->tcLossNotify            = TRUE;
	psrMyKeys->pciComPort              = 0;     // indicates no port selected
	psrMyKeys->pciNmeaVirtualComPort   = 0;     // indicates no port selected
	psrMyKeys->dwNmeaMessagesBitMap = 0;     // all zeroes, no NMEA messages being output
	psrMyKeys->dwNmeaBaudrate       = 9600;
	psrMyKeys->dwNmeaDataBits       = 8;
	psrMyKeys->dwNmeaParity         = NMEA_PARITY_NONE;     // kjf xxx yyy zzz what values should parity be???
	psrMyKeys->dwNmeaStopBits       = 1;
	psrMyKeys->dwNmeaComPortNumber  = 0;     // no output COM port entered

	// set up time zone defaults

	SetTzDescToUTC(psrMyKeys);

	// set defaults for undocumented settings

	//psrMyKeys->setTimeAggressive   = 0;
	//psrMyKeys->useCrudeTimeSet     = 0;
}

//
//		Name:			::VerifyRegistryKeys()
//
//		Description:	This will verify the keys exist in the registry.
//                      If the key does not exist, the key will be created and set to the default value.
//

void CSoftwareConfig::VerifyRegistryKeys()
{
	HKEY	paramKey;
	DWORD	paramType;
	TCHAR	tcParamPath[256];
	DWORD	paramSize;
	DWORD   dwDisposition;
	LONG    lStatus;

	REGISTRY_KEYS srDefaultKeys;
	REGISTRY_KEYS srKeysInRegistry;

	// ERROR_FILE_NOT_FOUND == 2

	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("Verify registry keys"));
	#endif

	GetKeyDefaults(&srDefaultKeys);

	memset(&srKeysInRegistry, 0, sizeof(srKeysInRegistry));

	// If the key does not exist it will be created

	GetRegistryPath(tcParamPath, sizeof(tcParamPath)/sizeof(TCHAR));

	wcscat_s(tcParamPath,sizeof(tcParamPath)/sizeof(TCHAR), _T("\\Parameters"));

	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE,
		                tcParamPath,
						0,
						NULL,
						0,
						KEY_ALL_ACCESS,
						NULL,
						&paramKey,
						&dwDisposition) != ERROR_SUCCESS)
	{
		// We had an error accessing the registry. We cannot recover from this error
		// so just exit.
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys path error"));
		#endif
		return;
	}

	// we do't care if the key was created or not. The verifies below will address this.
	//if (dwDisposition == REG_CREATED_NEW_KEY)
	//{
	//} else
	//if (dwDisposition == REG_OPENED_EXISTING_KEY)
	//{
	//} else
	//{
	//}


	paramType = REG_SZ;
	paramSize = sizeof(srKeysInRegistry.tzRegistryKey);

	lStatus = RegQueryValueEx(paramKey,
						REGISTRY_TIME_ZONE_KEY,
						NULL,
						&paramType,
						(BYTE *)srKeysInRegistry.tzRegistryKey,
						&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		// the key did not exist, set to default
		RegSetValueEx( paramKey,
							REGISTRY_TIME_ZONE_KEY,
							0,
							REG_SZ,
							(BYTE *)srDefaultKeys.tzRegistryKey,
							(DWORD)_tcslen(srDefaultKeys.tzRegistryKey)*sizeof(TCHAR) + sizeof(TCHAR));

		_tcscpy_s(m_srKeys.tzRegistryKey, srDefaultKeys.tzRegistryKey);

		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time zone key error"));
		#endif
	}

	paramType = REG_SZ;
	paramSize = sizeof(srKeysInRegistry.tzDesc);

	lStatus = RegQueryValueEx(paramKey,
						REGISTRY_TIME_ZONE_DESC,
						NULL,
						&paramType,
						(BYTE *)srKeysInRegistry.tzDesc,
						&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		// the key did not exist, set to default
		RegSetValueEx( paramKey,
							REGISTRY_TIME_ZONE_DESC,
							0,
							REG_SZ,
							(BYTE *)srDefaultKeys.tzDesc,
							(DWORD)_tcslen(srDefaultKeys.tzDesc)*sizeof(TCHAR) + sizeof(TCHAR));

		_tcscpy_s(m_srKeys.tzDesc, srDefaultKeys.tzDesc);

		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time zone desc error"));
		#endif
	}

	paramType = REG_BINARY;
	paramSize = sizeof(srDefaultKeys.tzInfo);

	lStatus = RegQueryValueEx(paramKey,
						REGISTRY_TIME_ZONE_INFO,
						NULL,
						&paramType,
						(BYTE *)&srKeysInRegistry.tzInfo,
						&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
							REGISTRY_TIME_ZONE_INFO,
							0,
							REG_BINARY,
							(BYTE *)&srDefaultKeys.tzInfo,
							sizeof(srDefaultKeys.tzInfo));

		m_srKeys.tzInfo = srDefaultKeys.tzInfo;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time zone info error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srDefaultKeys.dstEncoded);

	lStatus = RegQueryValueEx(paramKey,
					REGISTRY_DST_ENCODED,
					NULL,
					&paramType,
					(BYTE *)&srKeysInRegistry.dstEncoded,
					&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx( paramKey,
							REGISTRY_DST_ENCODED,
							0,
							REG_DWORD,
							(BYTE *)&srDefaultKeys.dstEncoded,
							sizeof(srDefaultKeys.dstEncoded));

		m_srKeys.dstEncoded = srDefaultKeys.dstEncoded;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys dst encoded error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srDefaultKeys.timeCodeSourceSmpte);

	lStatus = RegQueryValueEx(paramKey,
					REGISTRY_TIME_CODE_SOURCE_SMPTE,
					NULL,
					&paramType,
					(BYTE *)&srKeysInRegistry.timeCodeSourceSmpte,
					&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx( paramKey,
							REGISTRY_TIME_CODE_SOURCE_SMPTE,
							0,
							REG_DWORD,
							(BYTE *)&srDefaultKeys.timeCodeSourceSmpte,
							sizeof(srDefaultKeys.timeCodeSourceSmpte));

		m_srKeys.timeCodeSourceSmpte = srDefaultKeys.timeCodeSourceSmpte;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time code source SMPTE error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srDefaultKeys.timeCodeSourceSmpte);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_TIME_CODE_SOURCE_SMPTE,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.timeCodeSourceSmpte,
		&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx( paramKey,
			REGISTRY_TIME_CODE_SOURCE_SMPTE,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.timeCodeSourceSmpte,
			sizeof(srDefaultKeys.timeCodeSourceSmpte));

		m_srKeys.timeCodeSourceSmpte = srDefaultKeys.timeCodeSourceSmpte;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time code source SMPTE error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srDefaultKeys.timeCodeSourceIrig);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_TIME_CODE_SOURCE_IRIG,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.timeCodeSourceIrig,
		&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx( paramKey,
			REGISTRY_TIME_CODE_SOURCE_IRIG,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.timeCodeSourceIrig,
			sizeof(srDefaultKeys.timeCodeSourceIrig));

		m_srKeys.timeCodeSourceIrig = srDefaultKeys.timeCodeSourceIrig;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time code source IRIG error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.freewheelSynchronize);

	lStatus = RegQueryValueEx(paramKey,
					REGISTRY_FREEWHEEL_SYNC,
					NULL,
					&paramType,
					(BYTE *)&srKeysInRegistry.freewheelSynchronize,
					&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
							REGISTRY_FREEWHEEL_SYNC,
							0,
							REG_DWORD,
							(BYTE *)&srDefaultKeys.freewheelSynchronize,
							sizeof(srDefaultKeys.freewheelSynchronize));

		m_srKeys.freewheelSynchronize = srDefaultKeys.freewheelSynchronize;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys freewheel sync error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.freewheelDisplay);

	lStatus = RegQueryValueEx(paramKey,
					REGISTRY_FREEWHEEL_DISP,
					NULL,
					&paramType,
					(BYTE *)&srKeysInRegistry.freewheelDisplay,
					&paramSize);

	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
							REGISTRY_FREEWHEEL_DISP,
							0,
							REG_DWORD,
							(BYTE *)&srDefaultKeys.freewheelDisplay,
							sizeof(srDefaultKeys.freewheelDisplay));

		m_srKeys.freewheelDisplay = srDefaultKeys.freewheelDisplay;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys freewheel display error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.tcLossNotify);

	lStatus = RegQueryValueEx(paramKey,
					REGISTRY_TIME_CODE_LOSS,
					NULL,
					&paramType,
					(BYTE *)&srKeysInRegistry.tcLossNotify,
					&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
							REGISTRY_TIME_CODE_LOSS,
							0,
							REG_DWORD,
							(BYTE *)&srDefaultKeys.tcLossNotify,
							sizeof(srDefaultKeys.tcLossNotify));

		m_srKeys.tcLossNotify = srDefaultKeys.tcLossNotify;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys time code loss error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.pciComPort);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_PCI_COM_PORT,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.pciComPort,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_PCI_COM_PORT,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.pciComPort,
			sizeof(srDefaultKeys.pciComPort));

		m_srKeys.pciComPort = srDefaultKeys.pciComPort;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys PCIe COM port error"));
		#endif
	}

	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.pciNmeaVirtualComPort);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_PCI_VIRTUAL_COM_PORT,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.pciNmeaVirtualComPort,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_PCI_VIRTUAL_COM_PORT,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.pciNmeaVirtualComPort,
			sizeof(srDefaultKeys.pciNmeaVirtualComPort));

		m_srKeys.pciNmeaVirtualComPort = srDefaultKeys.pciNmeaVirtualComPort;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys PCIe Virtual COM port error"));
		#endif
	}


	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.dwNmeaMessagesBitMap);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_NMEA_MESSAGES_BIT_MAP,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.dwNmeaMessagesBitMap,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_NMEA_MESSAGES_BIT_MAP,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.dwNmeaMessagesBitMap,
			sizeof(srDefaultKeys.dwNmeaMessagesBitMap));

		m_srKeys.dwNmeaMessagesBitMap = srDefaultKeys.dwNmeaMessagesBitMap;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys NMEA messages bit map error"));
		#endif
	}


	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.dwNmeaBaudrate);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_NMEA_BAUD_RATE,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.dwNmeaBaudrate,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_NMEA_BAUD_RATE,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.dwNmeaBaudrate,
			sizeof(srDefaultKeys.dwNmeaBaudrate));

		m_srKeys.dwNmeaBaudrate = srDefaultKeys.dwNmeaBaudrate;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys NMEA baud rate error"));
		#endif
	}



	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.dwNmeaDataBits);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_NMEA_DATA_BITS,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.dwNmeaDataBits,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_NMEA_DATA_BITS,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.dwNmeaDataBits,
			sizeof(srDefaultKeys.dwNmeaDataBits));

		m_srKeys.dwNmeaDataBits = srDefaultKeys.dwNmeaDataBits;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys NMEA data bits error"));
		#endif
	}



	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.dwNmeaParity);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_NMEA_PARITY,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.dwNmeaParity,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_NMEA_PARITY,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.dwNmeaParity,
			sizeof(srDefaultKeys.dwNmeaParity));

		m_srKeys.dwNmeaParity = srDefaultKeys.dwNmeaParity;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys NMEA parity error"));
		#endif
	}



	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.dwNmeaStopBits);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_NMEA_STOP_BITS,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.dwNmeaStopBits,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_NMEA_STOP_BITS,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.dwNmeaStopBits,
			sizeof(srDefaultKeys.dwNmeaStopBits));

		m_srKeys.dwNmeaStopBits = srDefaultKeys.dwNmeaStopBits;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys NMEA stop bits error"));
		#endif
	}



	paramType = REG_DWORD;
	paramSize = sizeof(srKeysInRegistry.dwNmeaComPortNumber);

	lStatus = RegQueryValueEx(paramKey,
		REGISTRY_NMEA_COM_PORT_NUMBER,
		NULL,
		&paramType,
		(BYTE *)&srKeysInRegistry.dwNmeaComPortNumber,
		&paramSize);
	if (lStatus != ERROR_SUCCESS)
	{
		RegSetValueEx(	paramKey,
			REGISTRY_NMEA_COM_PORT_NUMBER,
			0,
			REG_DWORD,
			(BYTE *)&srDefaultKeys.dwNmeaComPortNumber,
			sizeof(srDefaultKeys.dwNmeaComPortNumber));

		m_srKeys.dwNmeaComPortNumber = srDefaultKeys.dwNmeaComPortNumber;
		#ifdef _DEBUG_ERROR
		cDebugConfig->DebugMsg(_T("!!!Verify config keys NMEA com port number error"));
		#endif
	}

	// note that the remaining configuration items are undocumented or special-purpose
	// configuration and configuration load does not fail if they are missing

	//paramType = REG_DWORD;
	//paramSize = sizeof(srKeysInRegistry.setTimeAggressive);

	//lStatus = RegQueryValueEx(paramKey,
	//				REGISTRY_AGGRESSIVE_TIME_SET,
	//				NULL,
	//				&paramType,
	//				(BYTE *)&srKeysInRegistry.setTimeAggressive,
	//				&paramSize);
	//if (lStatus == ERROR_FILE_NOT_FOUND)
	//{
	//	srDefaultKeys.setTimeAggressive = 0;
	//}

	//paramType = REG_DWORD;
	//paramSize = srKeysInRegistry.useCrudeTimeSet;

	//lStatus = RegQueryValueEx(paramKey,
	//				REGISTRY_CRUDE_TIME_SET,
	//				NULL,
	//				&paramType,
	//				(BYTE *)&srKeysInRegistry.useCrudeTimeSet,
	//				&paramSize);
	//if (lStatus == ERROR_FILE_NOT_FOUND)
	//{
	//	srDefaultKeys.useCrudeTimeSet = 0;
	//}

	RegCloseKey(paramKey);
}

void CSoftwareConfig::operator=(CSoftwareConfig &dest)
{
	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("config operator ="));
	#endif

	memcpy(m_srKeys.tzRegistryKey,dest.m_srKeys.tzRegistryKey,sizeof(m_srKeys.tzRegistryKey));
	memcpy(m_srKeys.tzDesc,dest.m_srKeys.tzDesc,sizeof(m_srKeys.tzDesc));
	memcpy(&m_srKeys.tzInfo,&dest.m_srKeys.tzInfo,sizeof(m_srKeys.tzInfo));

	m_srKeys.dstEncoded           = dest.m_srKeys.dstEncoded;
	m_srKeys.timeCodeSourceSmpte  = dest.m_srKeys.timeCodeSourceSmpte;
	m_srKeys.timeCodeSourceIrig   = dest.m_srKeys.timeCodeSourceIrig;
	m_srKeys.freewheelSynchronize = dest.m_srKeys.freewheelSynchronize;
	m_srKeys.freewheelDisplay     = dest.m_srKeys.freewheelDisplay;
	m_srKeys.tcLossNotify         = dest.m_srKeys.tcLossNotify;
	m_srKeys.pciComPort           = dest.m_srKeys.pciComPort;
	m_srKeys.pciNmeaVirtualComPort = dest.m_srKeys.pciNmeaVirtualComPort;

	m_srKeys.dwNmeaMessagesBitMap = dest.m_srKeys.dwNmeaMessagesBitMap;
	m_srKeys.dwNmeaBaudrate       = dest.m_srKeys.dwNmeaBaudrate;
	m_srKeys.dwNmeaDataBits       = dest.m_srKeys.dwNmeaDataBits;
	m_srKeys.dwNmeaParity         = dest.m_srKeys.dwNmeaParity;
	m_srKeys.dwNmeaStopBits       = dest.m_srKeys.dwNmeaStopBits;
	m_srKeys.dwNmeaComPortNumber  = dest.m_srKeys.dwNmeaComPortNumber;

	//m_srKeys.setTimeAggressive     = dest.m_srKeys.setTimeAggressive;
	//m_srKeys.useCrudeTimeSet       = dest.m_srKeys.useCrudeTimeSet;
}


//
//		Name:				::overloaded operator '=='
//
//		Description:	Compares two software configurations for equality
//
//		Returns:			TRUE if the software configurations compared are equal,
//							otherwise FALSE.
//

int CSoftwareConfig::operator==(CSoftwareConfig &tst)
{
	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("config operator =="));
	#endif

	//if ((!_tcsncmp(this->m_srKeys.tzRegistryKey,tst.m_srKeys.tzRegistryKey,sizeof(this->m_srKeys.tzRegistryKey)/sizeof(TCHAR))) &&
	//	(!_tcsncmp(this->m_srKeys.tzDesc,       tst.m_srKeys.tzDesc,       sizeof(this->m_srKeys.tzDesc)/sizeof(TCHAR)))        &&
	//	CompareTimeZoneDesc(&this->m_srKeys.tzInfo,&tst.m_srKeys.tzInfo)                                &&
	//	(this->m_srKeys.dstEncoded              == tst.m_srKeys.dstEncoded)                             &&
	//	(this->m_srKeys.timeCodeSource          == tst.m_srKeys.timeCodeSource)                         &&
	//	(this->m_srKeys.dwTimeCodeDateAvailable == tst.m_srKeys.dwTimeCodeDateAvailable)                &&
	//	(this->m_srKeys.freewheelSynchronize    == tst.m_srKeys.freewheelSynchronize)                   &&
	//	(this->m_srKeys.freewheelDisplay        == tst.m_srKeys.freewheelDisplay)                       &&
	//	(this->m_srKeys.tcLossNotify            == tst.m_srKeys.tcLossNotify)                           &&
	//	(this->m_srKeys.setTimeAggressive       == tst.m_srKeys.setTimeAggressive)                      &&
	//	(this->m_srKeys.useCrudeTimeSet         == tst.m_srKeys.useCrudeTimeSet))

	if ((!_tcsncmp(this->m_srKeys.tzRegistryKey,tst.m_srKeys.tzRegistryKey,sizeof(this->m_srKeys.tzRegistryKey)/sizeof(TCHAR))) &&
		(!_tcsncmp(this->m_srKeys.tzDesc,       tst.m_srKeys.tzDesc,       sizeof(this->m_srKeys.tzDesc)/sizeof(TCHAR)))        &&
		CompareTimeZoneDesc(&this->m_srKeys.tzInfo,&tst.m_srKeys.tzInfo)                             &&
		(this->m_srKeys.dstEncoded           == tst.m_srKeys.dstEncoded)                             &&
		(this->m_srKeys.timeCodeSourceSmpte  == tst.m_srKeys.timeCodeSourceSmpte)                    &&
		(this->m_srKeys.timeCodeSourceIrig   == tst.m_srKeys.timeCodeSourceIrig)                     &&
		(this->m_srKeys.freewheelSynchronize == tst.m_srKeys.freewheelSynchronize)                   &&
		(this->m_srKeys.freewheelDisplay     == tst.m_srKeys.freewheelDisplay)                       &&
		(this->m_srKeys.tcLossNotify         == tst.m_srKeys.tcLossNotify)                           &&
		(this->m_srKeys.pciComPort           == tst.m_srKeys.pciComPort)                                &&
		(this->m_srKeys.pciNmeaVirtualComPort == tst.m_srKeys.pciNmeaVirtualComPort)                    &&
		(this->m_srKeys.dwNmeaMessagesBitMap == tst.m_srKeys.dwNmeaMessagesBitMap)                      &&
		(this->m_srKeys.dwNmeaBaudrate       == tst.m_srKeys.dwNmeaBaudrate)                            &&
		(this->m_srKeys.dwNmeaDataBits       == tst.m_srKeys.dwNmeaDataBits)                            &&
		(this->m_srKeys.dwNmeaParity         == tst.m_srKeys.dwNmeaParity)                              &&
		(this->m_srKeys.dwNmeaStopBits       == tst.m_srKeys.dwNmeaStopBits)                            &&
		(this->m_srKeys.dwNmeaComPortNumber  == tst.m_srKeys.dwNmeaComPortNumber))
	{
		return(TRUE);
	}
	else
		return(FALSE);
}


//
//		Name:				::overloaded operator '!='
//
//		Description:	Compares two software configurations for inequality
//
//		Returns:			TRUE if the software configurations compared are NOT equal,
//							otherwise FALSE.
//

int CSoftwareConfig::operator!=(CSoftwareConfig &tst)
{
	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("config operator !="));
	#endif

	//if ((!_tcsncmp(this->m_srKeys.tzRegistryKey,tst.m_srKeys.tzRegistryKey,sizeof(this->m_srKeys.tzRegistryKey)/sizeof(TCHAR))) &&
	//	(!_tcsncmp(this->m_srKeys.tzDesc,       tst.m_srKeys.tzDesc,       sizeof(this->m_srKeys.tzDesc)/sizeof(TCHAR)))        &&
	//	CompareTimeZoneDesc(&this->m_srKeys.tzInfo,&tst.m_srKeys.tzInfo)                                &&
	//	(this->m_srKeys.dstEncoded              == tst.m_srKeys.dstEncoded)                             &&
	//	(this->m_srKeys.timeCodeSource          == tst.m_srKeys.timeCodeSource)                         &&
	//	(this->m_srKeys.dwTimeCodeDateAvailable == tst.m_srKeys.dwTimeCodeDateAvailable)                &&
	//	(this->m_srKeys.freewheelSynchronize    == tst.m_srKeys.freewheelSynchronize)                   &&
	//	(this->m_srKeys.freewheelDisplay        == tst.m_srKeys.freewheelDisplay)                       &&
	//	(this->m_srKeys.tcLossNotify            == tst.m_srKeys.tcLossNotify)                           &&
	//	(this->m_srKeys.setTimeAggressive       == tst.m_srKeys.setTimeAggressive)                      &&
	//	(this->m_srKeys.useCrudeTimeSet         == tst.m_srKeys.useCrudeTimeSet))

	if ((!_tcsncmp(this->m_srKeys.tzRegistryKey,tst.m_srKeys.tzRegistryKey,sizeof(this->m_srKeys.tzRegistryKey)/sizeof(TCHAR))) &&
		(!_tcsncmp(this->m_srKeys.tzDesc,       tst.m_srKeys.tzDesc,       sizeof(this->m_srKeys.tzDesc)/sizeof(TCHAR)))        &&
		CompareTimeZoneDesc(&this->m_srKeys.tzInfo,&tst.m_srKeys.tzInfo)                             &&
		(this->m_srKeys.dstEncoded           == tst.m_srKeys.dstEncoded)                             &&
		(this->m_srKeys.timeCodeSourceSmpte  == tst.m_srKeys.timeCodeSourceSmpte)                    &&
		(this->m_srKeys.timeCodeSourceIrig   == tst.m_srKeys.timeCodeSourceIrig)                     &&
		(this->m_srKeys.freewheelSynchronize == tst.m_srKeys.freewheelSynchronize)                   &&
		(this->m_srKeys.freewheelDisplay     == tst.m_srKeys.freewheelDisplay)                       &&
		(this->m_srKeys.tcLossNotify         == tst.m_srKeys.tcLossNotify)                           &&
		(this->m_srKeys.pciComPort           == tst.m_srKeys.pciComPort)                                &&
		(this->m_srKeys.pciNmeaVirtualComPort == tst.m_srKeys.pciNmeaVirtualComPort)                    &&
		(this->m_srKeys.dwNmeaMessagesBitMap == tst.m_srKeys.dwNmeaMessagesBitMap)                      &&
		(this->m_srKeys.dwNmeaBaudrate       == tst.m_srKeys.dwNmeaBaudrate)                            &&
		(this->m_srKeys.dwNmeaDataBits       == tst.m_srKeys.dwNmeaDataBits)                            &&
		(this->m_srKeys.dwNmeaParity         == tst.m_srKeys.dwNmeaParity)                              &&
		(this->m_srKeys.dwNmeaStopBits       == tst.m_srKeys.dwNmeaStopBits)                            &&
		(this->m_srKeys.dwNmeaComPortNumber  == tst.m_srKeys.dwNmeaComPortNumber))
	{
		return(FALSE);
	}
	else
		return(TRUE);
}


void CSoftwareConfig::SetTzDescToUTC(REGISTRY_KEYS *psrMyKeys)
{
	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("config set tz to utc"));
	#endif

	memset(psrMyKeys->tzRegistryKey,0,sizeof(psrMyKeys->tzRegistryKey));
	_tcscpy_s(psrMyKeys->tzRegistryKey,sizeof(psrMyKeys->tzRegistryKey)/sizeof(TCHAR),_T("GMT Standard Time"));

	memset(psrMyKeys->tzDesc,0,sizeof(psrMyKeys->tzDesc));
	_tcscpy_s(psrMyKeys->tzDesc,sizeof(psrMyKeys->tzDesc)/sizeof(TCHAR),_T("UTC (GMT)"));

	psrMyKeys->tzInfo.Bias = 0L;
	memset(psrMyKeys->tzInfo.StandardName,0,sizeof(psrMyKeys->tzInfo.StandardName));
	psrMyKeys->tzInfo.StandardBias = 0L;
	memset(&psrMyKeys->tzInfo.StandardDate,0,sizeof(psrMyKeys->tzInfo.StandardDate));
	memset(psrMyKeys->tzInfo.DaylightName,0,sizeof(psrMyKeys->tzInfo.DaylightName));
	psrMyKeys->tzInfo.DaylightBias = 0L;
	memset(&psrMyKeys->tzInfo.DaylightDate,0,sizeof(psrMyKeys->tzInfo.DaylightDate));
}


//
//		Name:				::IsTzDescUTC()
//
//		Description:	Determines if the tzInfo member is functionally a UTC/GMT
//							time zone description.
//

BOOL CSoftwareConfig::IsTzDescUTC()
{
	if (	(m_srKeys.tzInfo.Bias                == 0) &&
			(m_srKeys.tzInfo.StandardBias        == 0) &&
			(m_srKeys.tzInfo.DaylightBias        == 0) &&
			(m_srKeys.tzInfo.StandardDate.wMonth == 0) &&
			(m_srKeys.tzInfo.DaylightDate.wMonth == 0))
	{
		return(TRUE);
	}
	else
		return(FALSE);
}


//
//		Name:				::CompareTimeZoneDesc()
//
//		Description:	Compares two WCA_TIME_ZONE_INFORMATION structures for equality
//
//		Returns:			TRUE if structures are functionally equal, otherwise FALSE
//
//		Note:				Routine ignores the values of StandardName and DaylightName.
//

int CSoftwareConfig::CompareTimeZoneDesc(TIME_ZONE_INFORMATION *t1, TIME_ZONE_INFORMATION *t2)
{
	if (	(t1->Bias                       == t2->Bias) &&
			(t1->StandardBias               == t2->StandardBias) &&
			(t1->DaylightBias               == t2->DaylightBias) &&
			(t1->StandardDate.wMonth        == t2->StandardDate.wMonth) &&
			(t1->StandardDate.wDay          == t2->StandardDate.wDay) &&
			(t1->StandardDate.wYear         == t2->StandardDate.wYear) &&
			(t1->StandardDate.wHour         == t2->StandardDate.wHour) &&
			(t1->StandardDate.wMinute       == t2->StandardDate.wMinute) &&
			(t1->StandardDate.wSecond       == t2->StandardDate.wSecond) &&
			(t1->StandardDate.wMilliseconds == t2->StandardDate.wMilliseconds) &&
			(t1->DaylightDate.wMonth        == t2->DaylightDate.wMonth) &&
			(t1->DaylightDate.wDay          == t2->DaylightDate.wDay) &&
			(t1->DaylightDate.wYear         == t2->DaylightDate.wYear) &&
			(t1->DaylightDate.wHour         == t2->DaylightDate.wHour) &&
			(t1->DaylightDate.wMinute       == t2->DaylightDate.wMinute) &&
			(t1->DaylightDate.wSecond       == t2->DaylightDate.wSecond) &&
			(t1->DaylightDate.wMilliseconds == t2->DaylightDate.wMilliseconds))
	{
		return(TRUE);
	}
	else
		return(FALSE);
}


// This function is called by the API to convert the parameters received by the API
// into the parameters needed to save in the registry.
// The parameters are then saved to the registry.
//
// input: pointer to configuration values which need to be saved
//
// returns:: 0 if successful, else the parameter number which was invalid

UINT32 CSoftwareConfig::SaveConfigFromAPI(MCR_CONFIGURATION *pApiSetConfig)
{
	UINT32 ulReturnCode;
	int    ii;

	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("save config from API"));
	#endif

	ulReturnCode = CheckConfigFromAPI(pApiSetConfig);

	if (ulReturnCode)
	{
		// one or more of the parameters failed the check, return the error(s) without saving any values
		return ulReturnCode;
	}

	GetKeyDefaults(&m_srKeys);

	m_srKeys.dstEncoded           = pApiSetConfig->DstEncoded;
	m_srKeys.freewheelDisplay     = pApiSetConfig->FreewheelDisplay;
	m_srKeys.freewheelSynchronize = pApiSetConfig->FreewheelSynchronize;
	m_srKeys.tcLossNotify         = pApiSetConfig->TimeCodeLossNotify;
	m_srKeys.pciComPort           = pApiSetConfig->PciComPort;
	m_srKeys.pciNmeaVirtualComPort = pApiSetConfig->PciNmeaVirtualComPort;
	m_srKeys.timeCodeSourceSmpte  = pApiSetConfig->TimeCodeSourceSmpte;
	m_srKeys.timeCodeSourceIrig   = pApiSetConfig->TimeCodeSourceIrig;

	m_srKeys.tzInfo.Bias                    = pApiSetConfig->TimeZoneBiasSeconds;
	m_srKeys.tzInfo.StandardBias            = pApiSetConfig->StandardBiasSeconds;
	m_srKeys.tzInfo.StandardDate.wYear      = pApiSetConfig->StandardYear;
	m_srKeys.tzInfo.StandardDate.wMonth     = pApiSetConfig->StandardMonth;
	m_srKeys.tzInfo.StandardDate.wDayOfWeek = pApiSetConfig->StandardDayOfWeek;
	m_srKeys.tzInfo.StandardDate.wDay       = pApiSetConfig->StandardDate;
	m_srKeys.tzInfo.StandardDate.wHour      = pApiSetConfig->StandardHour;
	m_srKeys.tzInfo.StandardDate.wMinute    = pApiSetConfig->StandardMinute;
	m_srKeys.tzInfo.DaylightBias            = pApiSetConfig->DaylightBiasSeconds;
	m_srKeys.tzInfo.DaylightDate.wYear      = pApiSetConfig->DaylightYear;
	m_srKeys.tzInfo.DaylightDate.wMonth     = pApiSetConfig->DaylightMonth;
	m_srKeys.tzInfo.DaylightDate.wDayOfWeek = pApiSetConfig->DaylightDayOfWeek;
	m_srKeys.tzInfo.DaylightDate.wDay       = pApiSetConfig->DaylightDate;
	m_srKeys.tzInfo.DaylightDate.wHour      = pApiSetConfig->DaylightHour;
	m_srKeys.tzInfo.DaylightDate.wMinute    = pApiSetConfig->DaylightMinute;

	// Fill in the descriptions based on the values we received
	for (ii=0; ii<m_NumTimeZones; ii++)
	{
		if (m_srKeys.tzInfo.Bias == m_TimeZones[ii].tzInfo.Bias)
		{
			if ((m_srKeys.tzInfo.StandardBias == m_TimeZones[ii].tzInfo.StandardBias) &&
				(m_srKeys.tzInfo.DaylightBias == m_TimeZones[ii].tzInfo.DaylightBias))
			{
				if ((m_srKeys.tzInfo.StandardDate.wYear      == m_TimeZones[ii].tzInfo.StandardDate.wYear) &&
					(m_srKeys.tzInfo.StandardDate.wMonth     == m_TimeZones[ii].tzInfo.StandardDate.wMonth) &&
					(m_srKeys.tzInfo.StandardDate.wDay       == m_TimeZones[ii].tzInfo.StandardDate.wDay) &&
					(m_srKeys.tzInfo.StandardDate.wDayOfWeek == m_TimeZones[ii].tzInfo.StandardDate.wDayOfWeek) &&
					(m_srKeys.tzInfo.StandardDate.wHour      == m_TimeZones[ii].tzInfo.StandardDate.wHour) &&
					(m_srKeys.tzInfo.StandardDate.wMinute    == m_TimeZones[ii].tzInfo.StandardDate.wMinute))
				{
					if ((m_srKeys.tzInfo.DaylightDate.wYear      == m_TimeZones[ii].tzInfo.DaylightDate.wYear) &&
						(m_srKeys.tzInfo.DaylightDate.wMonth     == m_TimeZones[ii].tzInfo.DaylightDate.wMonth) &&
						(m_srKeys.tzInfo.DaylightDate.wDay       == m_TimeZones[ii].tzInfo.DaylightDate.wDay) &&
						(m_srKeys.tzInfo.DaylightDate.wDayOfWeek == m_TimeZones[ii].tzInfo.DaylightDate.wDayOfWeek) &&
						(m_srKeys.tzInfo.DaylightDate.wHour      == m_TimeZones[ii].tzInfo.DaylightDate.wHour) &&
						(m_srKeys.tzInfo.DaylightDate.wMinute    == m_TimeZones[ii].tzInfo.DaylightDate.wMinute))
					{
						// We have found the first time zone which matches what we received.
						// Save the description
						_tcscpy_s(m_srKeys.tzRegistryKey,sizeof(m_srKeys.tzRegistryKey)/sizeof(TCHAR),m_TimeZones[ii].tzAbbrev);
						_tcscpy_s(m_srKeys.tzDesc,sizeof(m_srKeys.tzDesc)/sizeof(TCHAR),m_TimeZones[ii].tzDesc);
						break;
					}
				}
			}
		}
	}

	m_srKeys.dwNmeaMessagesBitMap = pApiSetConfig->iNmeaMessagesBitMap;
	m_srKeys.dwNmeaBaudrate       = pApiSetConfig->iNmeaBaudrate;
	m_srKeys.dwNmeaDataBits       = pApiSetConfig->iNmeaDataBits;
	m_srKeys.dwNmeaParity         = pApiSetConfig->iNmeaParity;
	m_srKeys.dwNmeaStopBits       = pApiSetConfig->iNmeaStopBits;
	m_srKeys.dwNmeaComPortNumber  = pApiSetConfig->iNmeaComPortNumber;

	return 0;
}


// check the configuration
UINT32 CSoftwareConfig::CheckConfigFromAPI(MCR_CONFIGURATION *pApiSetConfig)
{
	UINT32 ulReturnCode;

	ulReturnCode = 0ul;

	if ((pApiSetConfig->DstEncoded < 0L) || (pApiSetConfig->DstEncoded > 1L))
		ulReturnCode |= INVALID_DST_ENCODED;

	if ((pApiSetConfig->FreewheelDisplay < 0L) || (pApiSetConfig->FreewheelDisplay > 1L))
		ulReturnCode |= INVALID_FREEWHEEL_DISPLAY;

	if ((pApiSetConfig->FreewheelSynchronize < 0L) || (pApiSetConfig->FreewheelSynchronize > 1L))
		ulReturnCode |= INVALID_FREEWHEEL_SYNCHRONIZE;

	if ((pApiSetConfig->TimeCodeLossNotify < 0L) || (pApiSetConfig->TimeCodeLossNotify > 1L))
		ulReturnCode |= INVALID_TIME_CODE_LOSS_NOTIFY;

	if ((pApiSetConfig->TimeCodeSourceSmpte != TC_SOURCE_DATE_NOT_INCLUDED) &&
		(pApiSetConfig->TimeCodeSourceSmpte != TC_SOURCE_DATE_LEITCH_FORMAT) &&
		(pApiSetConfig->TimeCodeSourceSmpte != TC_SOURCE_DATE_SMPTE_309M_WITH_TIME_ZONE) &&
		(pApiSetConfig->TimeCodeSourceSmpte != TC_SOURCE_DATE_SMPTE_309M_NO_TIME_ZONE))
		ulReturnCode |= INVALID_TIME_CODE_SOURCE;

	if ((pApiSetConfig->TimeCodeSourceIrig != TC_SOURCE_DATE_NOT_INCLUDED) &&
		(pApiSetConfig->TimeCodeSourceIrig != TC_SOURCE_DATE_DOY_YY) &&
		(pApiSetConfig->TimeCodeSourceIrig != TC_SOURCE_DATE_DOY))
		ulReturnCode |= INVALID_TIME_CODE_SOURCE;

	if ((pApiSetConfig->TimeZoneBiasSeconds < -90000L) || (pApiSetConfig->TimeZoneBiasSeconds > 90000L))   // 25 hours max
		ulReturnCode |= INVALID_TIME_ZONE_BIAS;

	if ((pApiSetConfig->StandardBiasSeconds < -7200L) || (pApiSetConfig->StandardBiasSeconds > 7200L))   // 2 hours max
		ulReturnCode |= INVALID_STANDARD_BIAS;

	if (pApiSetConfig->StandardYear)
	{
		if ((pApiSetConfig->StandardYear < 1900L) || (pApiSetConfig->StandardYear > 2200L))
			ulReturnCode |= INVALID_STANDARD_YEAR;
	}

	if ((pApiSetConfig->StandardMonth < 0L) || (pApiSetConfig->StandardMonth > 12L))
		ulReturnCode |= INVALID_STANDARD_MONTH;

	if ((pApiSetConfig->StandardDayOfWeek < 0L) || (pApiSetConfig->StandardDayOfWeek > 7L))
		ulReturnCode |= INVALID_STANDARD_DAY_OF_WEEK;

	if ((pApiSetConfig->StandardDate < 0L) || (pApiSetConfig->StandardDate > 31L))
		ulReturnCode |= INVALID_STANDARD_DATE;

	if ((pApiSetConfig->StandardHour < 0L) || (pApiSetConfig->StandardHour > 23L))
		ulReturnCode |= INVALID_STANDARD_HOUR;

	if ((pApiSetConfig->StandardMinute < 0L) || (pApiSetConfig->StandardMinute > 59L))
		ulReturnCode |= INVALID_STANDARD_MINUTE;

	if ((pApiSetConfig->DaylightBiasSeconds < -7200L) || (pApiSetConfig->DaylightBiasSeconds > 7200L))   // 2 hours max
		ulReturnCode |= INVALID_DAYLIGHT_BIAS;

	if (pApiSetConfig->DaylightYear)
	{
		if ((pApiSetConfig->DaylightYear < 1900L) || (pApiSetConfig->DaylightYear > 2200L))
			ulReturnCode |= INVALID_DAYLIGHT_YEAR;
	}

	if ((pApiSetConfig->DaylightMonth < 0L) || (pApiSetConfig->DaylightMonth > 12L))
		ulReturnCode |= INVALID_DAYLIGHT_MONTH;

	if ((pApiSetConfig->DaylightDayOfWeek < 0L) || (pApiSetConfig->DaylightDayOfWeek > 7L))
		ulReturnCode |= INVALID_DAYLIGHT_DAY_OF_WEEK;

	if ((pApiSetConfig->DaylightDate < 0L) || (pApiSetConfig->DaylightDate > 31L))
		ulReturnCode |= INVALID_DAYLIGHT_DATE;

	if ((pApiSetConfig->DaylightHour < 0L) || (pApiSetConfig->DaylightHour > 23L))
		ulReturnCode |= INVALID_DAYLIGHT_HOUR;

	if ((pApiSetConfig->DaylightMinute < 0L) || (pApiSetConfig->DaylightMinute > 59L))
		ulReturnCode |= INVALID_DAYLIGHT_MINUTE;

	// Any message bits set that we do not support?
	if (pApiSetConfig->iNmeaMessagesBitMap & (~NMEA_MESSAGE_MASK))
		ulReturnCode |= INVALID_NMEA_CONFIGURATION;

	if ((pApiSetConfig->iNmeaBaudrate != 4800)  &&
		(pApiSetConfig->iNmeaBaudrate != 9600)  &&
		(pApiSetConfig->iNmeaBaudrate != 19200) &&
		(pApiSetConfig->iNmeaBaudrate != 38400) &&
		(pApiSetConfig->iNmeaBaudrate != 57600) &&
		(pApiSetConfig->iNmeaBaudrate != 115200))
		ulReturnCode |= INVALID_NMEA_CONFIGURATION;

	if ((pApiSetConfig->iNmeaDataBits != 7)  &&
		(pApiSetConfig->iNmeaDataBits != 8))
		ulReturnCode |= INVALID_NMEA_CONFIGURATION;

	if ((pApiSetConfig->iNmeaParity != NMEA_PARITY_NONE)  &&
		(pApiSetConfig->iNmeaParity != NMEA_PARITY_ODD)  &&
		(pApiSetConfig->iNmeaParity != NMEA_PARITY_EVEN))
		ulReturnCode |= INVALID_NMEA_CONFIGURATION;

	if ((pApiSetConfig->iNmeaStopBits != 1)  &&
		(pApiSetConfig->iNmeaStopBits != 2))
		ulReturnCode |= INVALID_NMEA_CONFIGURATION;

	if ((pApiSetConfig->iNmeaComPortNumber < 0)  &&
		(pApiSetConfig->iNmeaComPortNumber > 1000))
		ulReturnCode |= INVALID_NMEA_CONFIGURATION;

	return ulReturnCode;
}


// This function is called by the API to convert the registry parameters into those
// used by the API.
// The converted parameters are then returned to the API.
//
// input: pointer to where to save the configuration values
//
// returns:: none

void CSoftwareConfig::GetConfigForAPI(MCR_CONFIGURATION *pApiGetConfig)
{
	#ifdef _DEBUG_CONFIG
	cDebugConfig->DebugMsg(_T("get config for API"));
	#endif

	pApiGetConfig->DstEncoded           = m_srKeys.dstEncoded;
	pApiGetConfig->FreewheelDisplay     = m_srKeys.freewheelDisplay;
	pApiGetConfig->FreewheelSynchronize = m_srKeys.freewheelSynchronize;
	pApiGetConfig->TimeCodeLossNotify   = m_srKeys.tcLossNotify;
	pApiGetConfig->PciComPort           = m_srKeys.pciComPort;
	pApiGetConfig->PciNmeaVirtualComPort = m_srKeys.pciNmeaVirtualComPort;
	pApiGetConfig->TimeCodeSourceSmpte  = m_srKeys.timeCodeSourceSmpte;
	pApiGetConfig->TimeCodeSourceIrig   = m_srKeys.timeCodeSourceIrig;

	pApiGetConfig->TimeZoneBiasSeconds  = m_srKeys.tzInfo.Bias;
	pApiGetConfig->StandardBiasSeconds  = m_srKeys.tzInfo.StandardBias;
	pApiGetConfig->StandardYear         = m_srKeys.tzInfo.StandardDate.wYear;
	pApiGetConfig->StandardMonth        = m_srKeys.tzInfo.StandardDate.wMonth;
	pApiGetConfig->StandardDayOfWeek    = m_srKeys.tzInfo.StandardDate.wDayOfWeek;
	pApiGetConfig->StandardDate         = m_srKeys.tzInfo.StandardDate.wDay;
	pApiGetConfig->StandardHour         = m_srKeys.tzInfo.StandardDate.wHour;
	pApiGetConfig->StandardMinute       = m_srKeys.tzInfo.StandardDate.wMinute;
	pApiGetConfig->DaylightBiasSeconds  = m_srKeys.tzInfo.DaylightBias;
	pApiGetConfig->DaylightYear         = m_srKeys.tzInfo.DaylightDate.wYear;
	pApiGetConfig->DaylightMonth        = m_srKeys.tzInfo.DaylightDate.wMonth;
	pApiGetConfig->DaylightDayOfWeek    = m_srKeys.tzInfo.DaylightDate.wDayOfWeek;
	pApiGetConfig->DaylightDate         = m_srKeys.tzInfo.DaylightDate.wDay;
	pApiGetConfig->DaylightHour         = m_srKeys.tzInfo.DaylightDate.wHour;
	pApiGetConfig->DaylightMinute       = m_srKeys.tzInfo.DaylightDate.wMinute;

	pApiGetConfig->iNmeaMessagesBitMap  = m_srKeys.dwNmeaMessagesBitMap;
	pApiGetConfig->iNmeaBaudrate        = m_srKeys.dwNmeaBaudrate;
	pApiGetConfig->iNmeaDataBits        = m_srKeys.dwNmeaDataBits;
	pApiGetConfig->iNmeaParity          = m_srKeys.dwNmeaParity;
	pApiGetConfig->iNmeaStopBits        = m_srKeys.dwNmeaStopBits;
	pApiGetConfig->iNmeaComPortNumber   = m_srKeys.dwNmeaComPortNumber;

}

//
//	Name:			::GetRegistryTimeZones()
//
//	Description:	Read all time zone information from registry into
//					array of TIME_ZONE_DESC
//

BOOL CSoftwareConfig::GetRegistryTimeZones(void)
{
	HKEY		enumKey, tzKey;
	TCHAR		tzKeyName[512];
	TCHAR		buffer[512];
	DWORD		tzKeyNameSize, tzDescKeyType, bufferSize;
	FILETIME	lastWrite;
	TCHAR		tzPath[256];
	LSTATUS     lStatus;

	if (GetVersion() & 0x80000000)
		_tcscpy_s(tzPath,sizeof(tzPath)/sizeof(TCHAR),_T("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Time Zones"));
	else
		_tcscpy_s(tzPath,sizeof(tzPath)/sizeof(TCHAR),_T("SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Time Zones"));

	RegOpenKeyEx(HKEY_LOCAL_MACHINE,tzPath,0,KEY_READ,&enumKey);

	m_NumTimeZones = 0;
	tzKeyNameSize = sizeof(tzKeyName)/sizeof(TCHAR);

	lStatus = RegEnumKeyEx(enumKey,m_NumTimeZones,tzKeyName,&tzKeyNameSize,NULL,NULL,NULL,&lastWrite);
	while(lStatus == ERROR_SUCCESS)
	{
		memset(m_TimeZones[m_NumTimeZones].tzAbbrev, 0, sizeof(m_TimeZones[m_NumTimeZones].tzAbbrev));

		_tcscpy_s(m_TimeZones[m_NumTimeZones].tzAbbrev, sizeof(m_TimeZones[m_NumTimeZones].tzAbbrev)/sizeof(TCHAR),tzKeyName);

		RegOpenKeyEx(enumKey,tzKeyName,0,KEY_READ,&tzKey);

		tzDescKeyType = REG_SZ;
		bufferSize = sizeof(buffer)/sizeof(TCHAR);

		RegQueryValueEx(tzKey,_T("Display"),NULL,&tzDescKeyType,(BYTE *)buffer,&bufferSize);

		memset(m_TimeZones[m_NumTimeZones].tzDesc, 0, sizeof(m_TimeZones[m_NumTimeZones].tzDesc));

		_tcscpy_s(m_TimeZones[m_NumTimeZones].tzDesc, sizeof(m_TimeZones[m_NumTimeZones].tzDesc)/sizeof(TCHAR),buffer);

		tzDescKeyType = REG_BINARY;
		bufferSize = sizeof(buffer);

		RegQueryValueEx(tzKey,_T("TZI"),NULL,&tzDescKeyType,(BYTE *)buffer,&bufferSize);

		memcpy(&m_TimeZones[m_NumTimeZones].tzInfo,buffer,sizeof(TIME_ZONE_DESC));

		RegCloseKey(tzKey);

		tzKeyNameSize = sizeof(tzKeyName)/sizeof(TCHAR);

		m_NumTimeZones++;

		// NOTE::: the following line is required for the MCR API, but must be commented out for the configuration utility!!!
		lStatus = RegEnumKeyEx(enumKey,m_NumTimeZones,tzKeyName,&tzKeyNameSize,NULL,NULL,NULL,&lastWrite);
	}

	// sort time zones according to bias

	qsort(&m_TimeZones[0],m_NumTimeZones,sizeof(TIME_ZONE_DESC),tzCompare);

	return TRUE;
}



//
//	Name:			tzCompare()
//
//	Description:	qsort comparison routine for sorting arrays of
//					TIME_ZONE_DESC structures by bias offset
//

int tzCompare(const void *arg1, const void *arg2)
{
	if (((TIME_ZONE_DESC *)arg1)->tzInfo.Bias < ((TIME_ZONE_DESC *)arg2)->tzInfo.Bias)
		return -1;

	if (((TIME_ZONE_DESC *)arg1)->tzInfo.Bias > ((TIME_ZONE_DESC *)arg2)->tzInfo.Bias)
		return 1;

	return 0;
}





//
//	Name:			::GetRegistrySerialPortNum()
//
//	Description:	returns the current serial port number
//

int CSoftwareConfig::GetRegistrySerialPortNum(void)
{
	return (int)m_srKeys.pciComPort;
}




//
//	Name:			::GetRegistryNmeaVirtualSerialPortNum()
//
//	Description:	returns the current NMEA Virtual serial port number
//                  assigned to the PCI card
//

int CSoftwareConfig::GetRegistryNmeaVirtualSerialPortNum(void)
{
	return (int)m_srKeys.pciNmeaVirtualComPort;
}



//
//	Name:			::GetRegistryNmeaSerialPortNum()
//
//	Description:	returns the current NMEA output serial port number
//                  selected by the user
//

int CSoftwareConfig::GetRegistryNmeaSerialPortNum(void)
{
	return (int)m_srKeys.dwNmeaComPortNumber;
}





//
//	Name:			::SetRegistrySerialPortNum()
//
//	Description:	Sets the current serial port number
//

BOOL CSoftwareConfig::SetRegistrySerialPortNum(int iSerialPortNumber)
{
	m_srKeys.pciComPort = iSerialPortNumber;

	return SaveConfigToRegistry();;
}






//
//	Name:			::SetRegistryNmeaVirtualSerialPortNum()
//
//	Description:	Sets the current NMEA virtual serial port number
//                  assigned to the PCI card.
//

BOOL CSoftwareConfig::SetRegistryNmeaVirtualSerialPortNum(int iSerialPortNumber)
{
	m_srKeys.pciNmeaVirtualComPort = iSerialPortNumber;
	m_srKeys.dwNmeaComPortNumber   = iSerialPortNumber;

	return SaveConfigToRegistry();;
}





//
//	Name:			::SetRegistryNmeaSerialPortNum()
//
//	Description:	Sets the current NMEA serial port number
//                  selected by the user.
//

BOOL CSoftwareConfig::SetRegistryNmeaSerialPortNum(int iSerialPortNumber)
{
	m_srKeys.dwNmeaComPortNumber = iSerialPortNumber;

	return SaveConfigToRegistry();;
}




