#pragma once


//#include "stdafx.h"


#define ishrpm(x)                      x > 11 ? 1 : 0 /* evals only 24-hr format */

//#ifdef _WIN32
//#include <windows.h>
//#include <winbase.h>
//#endif

#include "DebugOutput.h"

#pragma pack(push, 4)

#define READ_NAME             0x00000001
#define READ_BIAS             0x00000002
#define READ_STDBIAS          0x00000004
#define READ_DYLBIAS          0x00000008
#define READ_STDMONTH         0x00000010
#define READ_STDDAY           0x00000020
#define READ_STDYEAR          0x00000040
#define READ_STDDAYOFWEEK     0x00000080
#define READ_STDHOUR          0x00000100
#define READ_STDMINUTE        0x00000200
#define READ_STDSECOND        0x00000400
#define READ_STDMILLISECONDS  0x00000800
#define READ_DYLMONTH         0x00001000
#define READ_DYLDAY           0x00002000
#define READ_DYLYEAR          0x00004000
#define READ_DYLDAYOFWEEK     0x00008000
#define READ_DYLHOUR          0x00010000
#define READ_DYLMINUTE        0x00020000
#define READ_DYLSECOND        0x00040000
#define READ_DYLMILLISECONDS  0x00080000
#define READ_INVALIDATE       0x80000000
#define READ_ALL              0x000fffff

typedef struct _WCATIME {

   unsigned short wYear;
   unsigned short wMonth;
   unsigned short wDayOfWeek;
   unsigned short wDay;
   unsigned short wHour;
   unsigned short wMinute;
   unsigned short wSecond;
   unsigned short wMilliseconds;

	/* - if the wIsDstActive member is set to TRUE routines that expect WCATIME contents to be
	     a local time/date will consider the time to be in daylight savings time.
		- if the wIsDstActive member is set to FALSE routines that expect WCATIME contents to be
		  a local time/date will consider the time to be in standard time.
		- the UTCToLocalTime() routine will set this member
      - routines that do not differentiate between local time and UTC time will ignore this
		  member.
	 */

	unsigned short wIsDstActive;

} WCATIME;

/*
	EXPLANATION OF THE WCA_TIME_ZONE_INFORMATION STRUCTURE:
	-------------------------------------------------------

	[Bias]
	Specifies the current bias, in minutes, for local time translation.  The bias is the
	difference, in minutes, between Coordinated Universal Time (UTC) and local time.  All
	translations between UTC and local time are based on the following formula:

	UTC = local time + bias

	[StandardName]
	This field is for user use, and is ignored by the time zone library.

	[StandardDate]
	Specifies a WCATIME structure that contains a date and time when the transition from
	daylight savings time to standard time occurs.  If this date is not specified, the
	wMonth member in the WCATIME structure must be zero.  If this date is specified,
	the DaylightDate value in the WCA_TIME_ZONE_INFORMATION structure must also be specified.
	Local time translations done during the standard-time range are relative to the
	supplied StandardBias value.

   This member supports two date formats.  Absolute format specifies and exact date and time
	when standard time begins.  In this form, the wMonth, wDay, wHour, wMinute, wSecond,
	and wMilliseconds members of the WCATIME structure are used to specify an exact date.
	The wYear member should be set to 1 to indicate an absolute date.

	Day-in-month format is specified by setting the wMonth member to the desired month, setting
	the wDayOfWeek	member to an appropriate weekday (0=Sunday, etc.), and using a wDay value in
	the range 1 through 5 (1=first day of month, 2 = second day, 3 = third day, 4 = fourth day,
	5 = last day).  The wYear member should be set to 0 to indicate day-in-month format.

	[StandardBias]
	Specifies a bias value to be used during local time translations that occur during standard
	time. 	This value is added to the value of the Bias member to form the bias used during
	standard	time.  In most time zones, the value of this member is 0.

	[DaylightName]
	This field is for user use, and is ignored by the time zone library.

	[DaylightDate]
	Specified a WCATIME structure that contains a date and time when the transition from
	standard time to daylight savings time occurs.  If the date is not specified the wMonth
	member in the WCATIME structure must be set to zero.  If this date is specified, the
	StandardDate value in the WCA_TIME_ZONE_INFORMATION structure must also be specified.
	Local time translations during the daylight-time range are relative to the supplied
	DaylightBias value.  This member supports the absolute and day-in-month formats as
	described for the StandardDate member.

	[DaylightBias]
	Specifies a bias value to be used during local time translations that occur during daylight
	savings time.	This value is added to the value of the Bias member to form the bais used
	during daylight time translations.  The value of this member is normally -60.
*/

typedef struct _WCA_TIME_ZONE_INFORMATION {

   long     Bias;               /* in minutes */
   char     StandardName[64];
   WCATIME	StandardDate;		/* when daylight savings time ends */
   long     StandardBias;       /* in minutes */
   char     DaylightName[64];
   WCATIME	DaylightDate;       /* when daylight savings time starts */
   long     DaylightBias;		/* in minutes */

} WCA_TIME_ZONE_INFORMATION;

/* container for TIME_ZONE_INFORMATION with verbose time zone name */

typedef struct _TIME_ZONE_RECORD {
   
	unsigned short				id;                  
	char						tzName[256];
	WCA_TIME_ZONE_INFORMATION	tzInfo;

} TIME_ZONE_RECORD;


/* linked list node for TIME_ZONE_RECORD's */

typedef struct _TIME_ZONE_LIST {
   
   TIME_ZONE_RECORD        tzRec;
   unsigned short          filepos;
   void                    *next;
   void                    *prev;

} TIME_ZONE_LIST;


#pragma pack(pop)

class CTzLib
{
	public:
		CTzLib(CDebugOutput *cDebugOutputIn);
		~CTzLib();

		CDebugOutput *cDebugTzLib;

		unsigned char IsDSTDefined(WCA_TIME_ZONE_INFORMATION *);
		unsigned short LocalTimeToUTC(WCATIME *, WCA_TIME_ZONE_INFORMATION *, WCATIME *);
		unsigned short UTCToLocalTime(WCATIME *, WCA_TIME_ZONE_INFORMATION *, WCATIME *);
		void GetTimeDateFromSecondsSince1980(unsigned long, WCATIME *);
		unsigned long GetSecondsSince1980(WCATIME *);
		unsigned long GetDaysSince1980(WCATIME *, unsigned short);
		unsigned long GetSecondsSince1900(WCATIME *l);
		unsigned long GetDaysSince1900(WCATIME *, unsigned short);
		unsigned long BuildDSTSwitchTime(bool bIsDaylighSwitch, WCA_TIME_ZONE_INFORMATION *tz, WCATIME *currentUtcTime);
		int TimeDateCompare(WCATIME *, WCATIME *);
		int TimeCompare(WCATIME *, WCATIME *);
		unsigned short GetDaysInMonth(unsigned short, unsigned short);
		long GetDayCount(WCATIME *, unsigned short);
		unsigned char GetDayOfWeek(unsigned long);
		int IsLeapYear(unsigned short);
		unsigned short GetAmPmHour(unsigned short hr);
		unsigned short ConvertWindowYear(unsigned char year);
		unsigned short LoadTzFile(char *, unsigned short *, TIME_ZONE_LIST **);
		void FreeTzList(TIME_ZONE_LIST **);
		TIME_ZONE_LIST *AddTzToList(TIME_ZONE_LIST **);
		#ifdef _WIN32
		void Win32ToWCATimeConvert(WCATIME *pDest, SYSTEMTIME *pSrc);
		void Win32ToWCATimeZoneInfo(WCA_TIME_ZONE_INFORMATION *, TIME_ZONE_INFORMATION *);
		#endif
};


extern const unsigned short _daysMonthElapsedNonLeapYear[13];
extern const unsigned short _daysMonthElapsedLeapYear[13];
