#include <stdio.h>
#include <wtypes.h>
#include <winbase.h>
#include <winioctl.h>

#include "DebugOutput.h"

static char		_software[256];
static DWORD	_opflags;
static HANDLE	hDebugFile = NULL;
static HANDLE	hDebugFileMutex = NULL;

////////////////////////////////////////////////////////////////////////////////////////////
// Debugging support

BOOL DebugInit(char *sft, char *logFile, DWORD dwFlags)
{
	OSVERSIONINFO	osVer;

	hDebugFile      = NULL;
	hDebugFileMutex = NULL;

	osVer.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&osVer);

	_opflags = dwFlags;
	strcpy_s(_software,sizeof(_software),sft);


	if (_opflags & DEBUG_FILE)
	{
		if (!hDebugFile)
		{
			hDebugFile = CreateFile(logFile,
									GENERIC_READ | GENERIC_WRITE,
									FILE_SHARE_READ | FILE_SHARE_WRITE,
									NULL,
									OPEN_ALWAYS,
									FILE_ATTRIBUTE_NORMAL,
									NULL);
		}

		if (!hDebugFileMutex)
		{
			hDebugFileMutex = CreateMutex(NULL, FALSE, "Mutex_MC_DebugFile");
		}


		if (hDebugFile == INVALID_HANDLE_VALUE)
		{
			hDebugFile = NULL;

			return(FALSE);
		}


	if ((_opflags & DEBUG_NOHELLO) == 0)
	{
        #ifdef _DEBUG_INFO
		DebugMsg("************************************************");
		DebugMsg("Debugging Services initialized for %s",_software);
			DebugMsg(_software);
		DebugMsg("************************************************");
        #endif
	}

		return TRUE;

	} else
	{
		return FALSE;
	}
}


void DebugClose(void)
{
	if (hDebugFile)
		CloseHandle(hDebugFile);

	if (hDebugFileMutex)
		CloseHandle(hDebugFileMutex);
}

void DebugMsg(const char *pszFormat, ...)
{
	char	buf[1024];
	va_list arglist;
	ULONG	cbReturned;
	char    *pStart;
	int     iBytesLeft, iBytesUsed;

	if ((_opflags & DEBUG_NOLINEHEAD) == 0)
		sprintf_s(buf, sizeof(buf), "%s (%03lx): ",_software,(unsigned long)GetCurrentThreadId());
	else
		buf[0] = '\0';


	
	iBytesUsed = (int)strlen(buf);
	iBytesLeft = sizeof(buf) - iBytesUsed;
	pStart     = &buf[iBytesUsed];

	va_start(arglist, pszFormat);
	vsprintf_s(pStart, iBytesLeft, pszFormat, arglist);
	va_end(arglist);

	//strcat_s(buf, sizeof(buf), "\r\n");

	if (_opflags & DEBUG_FILE)
	{
		strcat_s(buf, sizeof(buf), "\r\n");

		WaitForSingleObject(hDebugFileMutex, INFINITE);
		SetFilePointer(hDebugFile,0,NULL,FILE_END);
		WriteFile(hDebugFile,buf,(int)strlen(buf),&cbReturned,NULL);
		FlushFileBuffers(hDebugFile);
		ReleaseMutex(hDebugFileMutex);
	}
}

BOOL DebugReset(void)
{

	if (!hDebugFile)
		return FALSE;

	if (_opflags & DEBUG_FILE)
	{
		SetFilePointer(hDebugFile,0,NULL,FILE_BEGIN);
		SetEndOfFile(hDebugFile);

		return(TRUE);
	}
	else
		return(FALSE);
}